using System;
using System.Data;
using System.Linq;
using System.Linq.Dynamic.Core;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Text.Encodings.Web;
using Microsoft.AspNetCore.Components;
using Microsoft.Data.SqlClient;
using Microsoft.EntityFrameworkCore;
using Radzen;

using Net8AutoSecurityExport.Data;

namespace Net8AutoSecurityExport
{
    public partial class NorthwindService
    {
        NorthwindContext Context
        {
           get
           {
             return this.context;
           }
        }

        private readonly NorthwindContext context;
        private readonly NavigationManager navigationManager;

        public NorthwindService(NorthwindContext context, NavigationManager navigationManager)
        {
            this.context = context;
            this.navigationManager = navigationManager;
        }

        public void Reset() => Context.ChangeTracker.Entries().Where(e => e.Entity != null).ToList().ForEach(e => e.State = EntityState.Detached);

        public void ApplyQuery<T>(ref IQueryable<T> items, Query query = null)
        {
            if (query != null)
            {
                if (!string.IsNullOrEmpty(query.Filter))
                {
                    if (query.FilterParameters != null)
                    {
                        items = items.Where(query.Filter, query.FilterParameters);
                    }
                    else
                    {
                        items = items.Where(query.Filter);
                    }
                }

                if (!string.IsNullOrEmpty(query.OrderBy))
                {
                    items = items.OrderBy(query.OrderBy);
                }

                if (query.Skip.HasValue)
                {
                    items = items.Skip(query.Skip.Value);
                }

                if (query.Top.HasValue)
                {
                    items = items.Take(query.Top.Value);
                }
            }
        }


        public async Task ExportCategoriesToExcel(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/categories/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/categories/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        public async Task ExportCategoriesToCSV(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/categories/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/categories/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        partial void OnCategoriesRead(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Category> items);

        public async Task<IQueryable<Net8AutoSecurityExport.Models.Northwind.Category>> GetCategories(Query query = null)
        {
            var items = Context.Categories.AsQueryable();


            if (query != null)
            {
                if (!string.IsNullOrEmpty(query.Expand))
                {
                    var propertiesToExpand = query.Expand.Split(',');
                    foreach(var p in propertiesToExpand)
                    {
                        items = items.Include(p.Trim());
                    }
                }

                ApplyQuery(ref items, query);
            }

            OnCategoriesRead(ref items);

            return await Task.FromResult(items);
        }

        partial void OnCategoryGet(Net8AutoSecurityExport.Models.Northwind.Category item);
        partial void OnGetCategoryByCategoryId(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Category> items);


        public async Task<Net8AutoSecurityExport.Models.Northwind.Category> GetCategoryByCategoryId(int categoryid)
        {
            var items = Context.Categories
                              .AsNoTracking()
                              .Where(i => i.CategoryID == categoryid);

 
            OnGetCategoryByCategoryId(ref items);

            var itemToReturn = items.FirstOrDefault();

            OnCategoryGet(itemToReturn);

            return await Task.FromResult(itemToReturn);
        }

        partial void OnCategoryCreated(Net8AutoSecurityExport.Models.Northwind.Category item);
        partial void OnAfterCategoryCreated(Net8AutoSecurityExport.Models.Northwind.Category item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Category> CreateCategory(Net8AutoSecurityExport.Models.Northwind.Category category)
        {
            OnCategoryCreated(category);

            var existingItem = Context.Categories
                              .Where(i => i.CategoryID == category.CategoryID)
                              .FirstOrDefault();

            if (existingItem != null)
            {
               throw new Exception("Item already available");
            }            

            try
            {
                Context.Categories.Add(category);
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(category).State = EntityState.Detached;
                throw;
            }

            OnAfterCategoryCreated(category);

            return category;
        }

        public async Task<Net8AutoSecurityExport.Models.Northwind.Category> CancelCategoryChanges(Net8AutoSecurityExport.Models.Northwind.Category item)
        {
            var entityToCancel = Context.Entry(item);
            if (entityToCancel.State == EntityState.Modified)
            {
              entityToCancel.CurrentValues.SetValues(entityToCancel.OriginalValues);
              entityToCancel.State = EntityState.Unchanged;
            }

            return item;
        }

        partial void OnCategoryUpdated(Net8AutoSecurityExport.Models.Northwind.Category item);
        partial void OnAfterCategoryUpdated(Net8AutoSecurityExport.Models.Northwind.Category item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Category> UpdateCategory(int categoryid, Net8AutoSecurityExport.Models.Northwind.Category category)
        {
            OnCategoryUpdated(category);

            var itemToUpdate = Context.Categories
                              .Where(i => i.CategoryID == category.CategoryID)
                              .FirstOrDefault();

            if (itemToUpdate == null)
            {
               throw new Exception("Item no longer available");
            }
                
            var entryToUpdate = Context.Entry(itemToUpdate);
            entryToUpdate.CurrentValues.SetValues(category);
            entryToUpdate.State = EntityState.Modified;

            Context.SaveChanges();

            OnAfterCategoryUpdated(category);

            return category;
        }

        partial void OnCategoryDeleted(Net8AutoSecurityExport.Models.Northwind.Category item);
        partial void OnAfterCategoryDeleted(Net8AutoSecurityExport.Models.Northwind.Category item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Category> DeleteCategory(int categoryid)
        {
            var itemToDelete = Context.Categories
                              .Where(i => i.CategoryID == categoryid)
                              .Include(i => i.Products)
                              .FirstOrDefault();

            if (itemToDelete == null)
            {
               throw new Exception("Item no longer available");
            }

            OnCategoryDeleted(itemToDelete);


            Context.Categories.Remove(itemToDelete);

            try
            {
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(itemToDelete).State = EntityState.Unchanged;
                throw;
            }

            OnAfterCategoryDeleted(itemToDelete);

            return itemToDelete;
        }
    
        public async Task ExportCustomerCustomerDemosToExcel(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/customercustomerdemos/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/customercustomerdemos/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        public async Task ExportCustomerCustomerDemosToCSV(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/customercustomerdemos/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/customercustomerdemos/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        partial void OnCustomerCustomerDemosRead(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo> items);

        public async Task<IQueryable<Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo>> GetCustomerCustomerDemos(Query query = null)
        {
            var items = Context.CustomerCustomerDemos.AsQueryable();

            items = items.Include(i => i.Customer);
            items = items.Include(i => i.CustomerDemographic);

            if (query != null)
            {
                if (!string.IsNullOrEmpty(query.Expand))
                {
                    var propertiesToExpand = query.Expand.Split(',');
                    foreach(var p in propertiesToExpand)
                    {
                        items = items.Include(p.Trim());
                    }
                }

                ApplyQuery(ref items, query);
            }

            OnCustomerCustomerDemosRead(ref items);

            return await Task.FromResult(items);
        }

        partial void OnCustomerCustomerDemoGet(Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo item);
        partial void OnGetCustomerCustomerDemoByCustomerIdAndCustomerTypeId(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo> items);


        public async Task<Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo> GetCustomerCustomerDemoByCustomerIdAndCustomerTypeId(string customerid, string customertypeid)
        {
            var items = Context.CustomerCustomerDemos
                              .AsNoTracking()
                              .Where(i => i.CustomerID == customerid && i.CustomerTypeID == customertypeid);

            items = items.Include(i => i.Customer);
            items = items.Include(i => i.CustomerDemographic);
 
            OnGetCustomerCustomerDemoByCustomerIdAndCustomerTypeId(ref items);

            var itemToReturn = items.FirstOrDefault();

            OnCustomerCustomerDemoGet(itemToReturn);

            return await Task.FromResult(itemToReturn);
        }

        partial void OnCustomerCustomerDemoCreated(Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo item);
        partial void OnAfterCustomerCustomerDemoCreated(Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo> CreateCustomerCustomerDemo(Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo customercustomerdemo)
        {
            OnCustomerCustomerDemoCreated(customercustomerdemo);

            var existingItem = Context.CustomerCustomerDemos
                              .Where(i => i.CustomerID == customercustomerdemo.CustomerID && i.CustomerTypeID == customercustomerdemo.CustomerTypeID)
                              .FirstOrDefault();

            if (existingItem != null)
            {
               throw new Exception("Item already available");
            }            

            try
            {
                Context.CustomerCustomerDemos.Add(customercustomerdemo);
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(customercustomerdemo).State = EntityState.Detached;
                throw;
            }

            OnAfterCustomerCustomerDemoCreated(customercustomerdemo);

            return customercustomerdemo;
        }

        public async Task<Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo> CancelCustomerCustomerDemoChanges(Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo item)
        {
            var entityToCancel = Context.Entry(item);
            if (entityToCancel.State == EntityState.Modified)
            {
              entityToCancel.CurrentValues.SetValues(entityToCancel.OriginalValues);
              entityToCancel.State = EntityState.Unchanged;
            }

            return item;
        }

        partial void OnCustomerCustomerDemoUpdated(Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo item);
        partial void OnAfterCustomerCustomerDemoUpdated(Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo> UpdateCustomerCustomerDemo(string customerid, string customertypeid, Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo customercustomerdemo)
        {
            OnCustomerCustomerDemoUpdated(customercustomerdemo);

            var itemToUpdate = Context.CustomerCustomerDemos
                              .Where(i => i.CustomerID == customercustomerdemo.CustomerID && i.CustomerTypeID == customercustomerdemo.CustomerTypeID)
                              .FirstOrDefault();

            if (itemToUpdate == null)
            {
               throw new Exception("Item no longer available");
            }
                
            var entryToUpdate = Context.Entry(itemToUpdate);
            entryToUpdate.CurrentValues.SetValues(customercustomerdemo);
            entryToUpdate.State = EntityState.Modified;

            Context.SaveChanges();

            OnAfterCustomerCustomerDemoUpdated(customercustomerdemo);

            return customercustomerdemo;
        }

        partial void OnCustomerCustomerDemoDeleted(Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo item);
        partial void OnAfterCustomerCustomerDemoDeleted(Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.CustomerCustomerDemo> DeleteCustomerCustomerDemo(string customerid, string customertypeid)
        {
            var itemToDelete = Context.CustomerCustomerDemos
                              .Where(i => i.CustomerID == customerid && i.CustomerTypeID == customertypeid)
                              .FirstOrDefault();

            if (itemToDelete == null)
            {
               throw new Exception("Item no longer available");
            }

            OnCustomerCustomerDemoDeleted(itemToDelete);


            Context.CustomerCustomerDemos.Remove(itemToDelete);

            try
            {
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(itemToDelete).State = EntityState.Unchanged;
                throw;
            }

            OnAfterCustomerCustomerDemoDeleted(itemToDelete);

            return itemToDelete;
        }
    
        public async Task ExportCustomerDemographicsToExcel(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/customerdemographics/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/customerdemographics/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        public async Task ExportCustomerDemographicsToCSV(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/customerdemographics/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/customerdemographics/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        partial void OnCustomerDemographicsRead(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.CustomerDemographic> items);

        public async Task<IQueryable<Net8AutoSecurityExport.Models.Northwind.CustomerDemographic>> GetCustomerDemographics(Query query = null)
        {
            var items = Context.CustomerDemographics.AsQueryable();


            if (query != null)
            {
                if (!string.IsNullOrEmpty(query.Expand))
                {
                    var propertiesToExpand = query.Expand.Split(',');
                    foreach(var p in propertiesToExpand)
                    {
                        items = items.Include(p.Trim());
                    }
                }

                ApplyQuery(ref items, query);
            }

            OnCustomerDemographicsRead(ref items);

            return await Task.FromResult(items);
        }

        partial void OnCustomerDemographicGet(Net8AutoSecurityExport.Models.Northwind.CustomerDemographic item);
        partial void OnGetCustomerDemographicByCustomerTypeId(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.CustomerDemographic> items);


        public async Task<Net8AutoSecurityExport.Models.Northwind.CustomerDemographic> GetCustomerDemographicByCustomerTypeId(string customertypeid)
        {
            var items = Context.CustomerDemographics
                              .AsNoTracking()
                              .Where(i => i.CustomerTypeID == customertypeid);

 
            OnGetCustomerDemographicByCustomerTypeId(ref items);

            var itemToReturn = items.FirstOrDefault();

            OnCustomerDemographicGet(itemToReturn);

            return await Task.FromResult(itemToReturn);
        }

        partial void OnCustomerDemographicCreated(Net8AutoSecurityExport.Models.Northwind.CustomerDemographic item);
        partial void OnAfterCustomerDemographicCreated(Net8AutoSecurityExport.Models.Northwind.CustomerDemographic item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.CustomerDemographic> CreateCustomerDemographic(Net8AutoSecurityExport.Models.Northwind.CustomerDemographic customerdemographic)
        {
            OnCustomerDemographicCreated(customerdemographic);

            var existingItem = Context.CustomerDemographics
                              .Where(i => i.CustomerTypeID == customerdemographic.CustomerTypeID)
                              .FirstOrDefault();

            if (existingItem != null)
            {
               throw new Exception("Item already available");
            }            

            try
            {
                Context.CustomerDemographics.Add(customerdemographic);
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(customerdemographic).State = EntityState.Detached;
                throw;
            }

            OnAfterCustomerDemographicCreated(customerdemographic);

            return customerdemographic;
        }

        public async Task<Net8AutoSecurityExport.Models.Northwind.CustomerDemographic> CancelCustomerDemographicChanges(Net8AutoSecurityExport.Models.Northwind.CustomerDemographic item)
        {
            var entityToCancel = Context.Entry(item);
            if (entityToCancel.State == EntityState.Modified)
            {
              entityToCancel.CurrentValues.SetValues(entityToCancel.OriginalValues);
              entityToCancel.State = EntityState.Unchanged;
            }

            return item;
        }

        partial void OnCustomerDemographicUpdated(Net8AutoSecurityExport.Models.Northwind.CustomerDemographic item);
        partial void OnAfterCustomerDemographicUpdated(Net8AutoSecurityExport.Models.Northwind.CustomerDemographic item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.CustomerDemographic> UpdateCustomerDemographic(string customertypeid, Net8AutoSecurityExport.Models.Northwind.CustomerDemographic customerdemographic)
        {
            OnCustomerDemographicUpdated(customerdemographic);

            var itemToUpdate = Context.CustomerDemographics
                              .Where(i => i.CustomerTypeID == customerdemographic.CustomerTypeID)
                              .FirstOrDefault();

            if (itemToUpdate == null)
            {
               throw new Exception("Item no longer available");
            }
                
            var entryToUpdate = Context.Entry(itemToUpdate);
            entryToUpdate.CurrentValues.SetValues(customerdemographic);
            entryToUpdate.State = EntityState.Modified;

            Context.SaveChanges();

            OnAfterCustomerDemographicUpdated(customerdemographic);

            return customerdemographic;
        }

        partial void OnCustomerDemographicDeleted(Net8AutoSecurityExport.Models.Northwind.CustomerDemographic item);
        partial void OnAfterCustomerDemographicDeleted(Net8AutoSecurityExport.Models.Northwind.CustomerDemographic item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.CustomerDemographic> DeleteCustomerDemographic(string customertypeid)
        {
            var itemToDelete = Context.CustomerDemographics
                              .Where(i => i.CustomerTypeID == customertypeid)
                              .Include(i => i.CustomerCustomerDemos)
                              .FirstOrDefault();

            if (itemToDelete == null)
            {
               throw new Exception("Item no longer available");
            }

            OnCustomerDemographicDeleted(itemToDelete);


            Context.CustomerDemographics.Remove(itemToDelete);

            try
            {
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(itemToDelete).State = EntityState.Unchanged;
                throw;
            }

            OnAfterCustomerDemographicDeleted(itemToDelete);

            return itemToDelete;
        }
    
        public async Task ExportCustomersToExcel(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/customers/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/customers/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        public async Task ExportCustomersToCSV(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/customers/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/customers/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        partial void OnCustomersRead(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Customer> items);

        public async Task<IQueryable<Net8AutoSecurityExport.Models.Northwind.Customer>> GetCustomers(Query query = null)
        {
            var items = Context.Customers.AsQueryable();


            if (query != null)
            {
                if (!string.IsNullOrEmpty(query.Expand))
                {
                    var propertiesToExpand = query.Expand.Split(',');
                    foreach(var p in propertiesToExpand)
                    {
                        items = items.Include(p.Trim());
                    }
                }

                ApplyQuery(ref items, query);
            }

            OnCustomersRead(ref items);

            return await Task.FromResult(items);
        }

        partial void OnCustomerGet(Net8AutoSecurityExport.Models.Northwind.Customer item);
        partial void OnGetCustomerByCustomerId(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Customer> items);


        public async Task<Net8AutoSecurityExport.Models.Northwind.Customer> GetCustomerByCustomerId(string customerid)
        {
            var items = Context.Customers
                              .AsNoTracking()
                              .Where(i => i.CustomerID == customerid);

 
            OnGetCustomerByCustomerId(ref items);

            var itemToReturn = items.FirstOrDefault();

            OnCustomerGet(itemToReturn);

            return await Task.FromResult(itemToReturn);
        }

        partial void OnCustomerCreated(Net8AutoSecurityExport.Models.Northwind.Customer item);
        partial void OnAfterCustomerCreated(Net8AutoSecurityExport.Models.Northwind.Customer item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Customer> CreateCustomer(Net8AutoSecurityExport.Models.Northwind.Customer customer)
        {
            OnCustomerCreated(customer);

            var existingItem = Context.Customers
                              .Where(i => i.CustomerID == customer.CustomerID)
                              .FirstOrDefault();

            if (existingItem != null)
            {
               throw new Exception("Item already available");
            }            

            try
            {
                Context.Customers.Add(customer);
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(customer).State = EntityState.Detached;
                throw;
            }

            OnAfterCustomerCreated(customer);

            return customer;
        }

        public async Task<Net8AutoSecurityExport.Models.Northwind.Customer> CancelCustomerChanges(Net8AutoSecurityExport.Models.Northwind.Customer item)
        {
            var entityToCancel = Context.Entry(item);
            if (entityToCancel.State == EntityState.Modified)
            {
              entityToCancel.CurrentValues.SetValues(entityToCancel.OriginalValues);
              entityToCancel.State = EntityState.Unchanged;
            }

            return item;
        }

        partial void OnCustomerUpdated(Net8AutoSecurityExport.Models.Northwind.Customer item);
        partial void OnAfterCustomerUpdated(Net8AutoSecurityExport.Models.Northwind.Customer item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Customer> UpdateCustomer(string customerid, Net8AutoSecurityExport.Models.Northwind.Customer customer)
        {
            OnCustomerUpdated(customer);

            var itemToUpdate = Context.Customers
                              .Where(i => i.CustomerID == customer.CustomerID)
                              .FirstOrDefault();

            if (itemToUpdate == null)
            {
               throw new Exception("Item no longer available");
            }
                
            var entryToUpdate = Context.Entry(itemToUpdate);
            entryToUpdate.CurrentValues.SetValues(customer);
            entryToUpdate.State = EntityState.Modified;

            Context.SaveChanges();

            OnAfterCustomerUpdated(customer);

            return customer;
        }

        partial void OnCustomerDeleted(Net8AutoSecurityExport.Models.Northwind.Customer item);
        partial void OnAfterCustomerDeleted(Net8AutoSecurityExport.Models.Northwind.Customer item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Customer> DeleteCustomer(string customerid)
        {
            var itemToDelete = Context.Customers
                              .Where(i => i.CustomerID == customerid)
                              .Include(i => i.CustomerCustomerDemos)
                              .Include(i => i.Orders)
                              .FirstOrDefault();

            if (itemToDelete == null)
            {
               throw new Exception("Item no longer available");
            }

            OnCustomerDeleted(itemToDelete);


            Context.Customers.Remove(itemToDelete);

            try
            {
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(itemToDelete).State = EntityState.Unchanged;
                throw;
            }

            OnAfterCustomerDeleted(itemToDelete);

            return itemToDelete;
        }
    
        public async Task ExportEmployeesToExcel(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/employees/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/employees/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        public async Task ExportEmployeesToCSV(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/employees/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/employees/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        partial void OnEmployeesRead(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Employee> items);

        public async Task<IQueryable<Net8AutoSecurityExport.Models.Northwind.Employee>> GetEmployees(Query query = null)
        {
            var items = Context.Employees.AsQueryable();

            items = items.Include(i => i.Employee1);

            if (query != null)
            {
                if (!string.IsNullOrEmpty(query.Expand))
                {
                    var propertiesToExpand = query.Expand.Split(',');
                    foreach(var p in propertiesToExpand)
                    {
                        items = items.Include(p.Trim());
                    }
                }

                ApplyQuery(ref items, query);
            }

            OnEmployeesRead(ref items);

            return await Task.FromResult(items);
        }

        partial void OnEmployeeGet(Net8AutoSecurityExport.Models.Northwind.Employee item);
        partial void OnGetEmployeeByEmployeeId(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Employee> items);


        public async Task<Net8AutoSecurityExport.Models.Northwind.Employee> GetEmployeeByEmployeeId(int employeeid)
        {
            var items = Context.Employees
                              .AsNoTracking()
                              .Where(i => i.EmployeeID == employeeid);

            items = items.Include(i => i.Employee1);
 
            OnGetEmployeeByEmployeeId(ref items);

            var itemToReturn = items.FirstOrDefault();

            OnEmployeeGet(itemToReturn);

            return await Task.FromResult(itemToReturn);
        }

        partial void OnEmployeeCreated(Net8AutoSecurityExport.Models.Northwind.Employee item);
        partial void OnAfterEmployeeCreated(Net8AutoSecurityExport.Models.Northwind.Employee item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Employee> CreateEmployee(Net8AutoSecurityExport.Models.Northwind.Employee employee)
        {
            OnEmployeeCreated(employee);

            var existingItem = Context.Employees
                              .Where(i => i.EmployeeID == employee.EmployeeID)
                              .FirstOrDefault();

            if (existingItem != null)
            {
               throw new Exception("Item already available");
            }            

            try
            {
                Context.Employees.Add(employee);
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(employee).State = EntityState.Detached;
                throw;
            }

            OnAfterEmployeeCreated(employee);

            return employee;
        }

        public async Task<Net8AutoSecurityExport.Models.Northwind.Employee> CancelEmployeeChanges(Net8AutoSecurityExport.Models.Northwind.Employee item)
        {
            var entityToCancel = Context.Entry(item);
            if (entityToCancel.State == EntityState.Modified)
            {
              entityToCancel.CurrentValues.SetValues(entityToCancel.OriginalValues);
              entityToCancel.State = EntityState.Unchanged;
            }

            return item;
        }

        partial void OnEmployeeUpdated(Net8AutoSecurityExport.Models.Northwind.Employee item);
        partial void OnAfterEmployeeUpdated(Net8AutoSecurityExport.Models.Northwind.Employee item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Employee> UpdateEmployee(int employeeid, Net8AutoSecurityExport.Models.Northwind.Employee employee)
        {
            OnEmployeeUpdated(employee);

            var itemToUpdate = Context.Employees
                              .Where(i => i.EmployeeID == employee.EmployeeID)
                              .FirstOrDefault();

            if (itemToUpdate == null)
            {
               throw new Exception("Item no longer available");
            }
                
            var entryToUpdate = Context.Entry(itemToUpdate);
            entryToUpdate.CurrentValues.SetValues(employee);
            entryToUpdate.State = EntityState.Modified;

            Context.SaveChanges();

            OnAfterEmployeeUpdated(employee);

            return employee;
        }

        partial void OnEmployeeDeleted(Net8AutoSecurityExport.Models.Northwind.Employee item);
        partial void OnAfterEmployeeDeleted(Net8AutoSecurityExport.Models.Northwind.Employee item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Employee> DeleteEmployee(int employeeid)
        {
            var itemToDelete = Context.Employees
                              .Where(i => i.EmployeeID == employeeid)
                              .Include(i => i.Employees1)
                              .Include(i => i.EmployeeTerritories)
                              .Include(i => i.Orders)
                              .FirstOrDefault();

            if (itemToDelete == null)
            {
               throw new Exception("Item no longer available");
            }

            OnEmployeeDeleted(itemToDelete);


            Context.Employees.Remove(itemToDelete);

            try
            {
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(itemToDelete).State = EntityState.Unchanged;
                throw;
            }

            OnAfterEmployeeDeleted(itemToDelete);

            return itemToDelete;
        }
    
        public async Task ExportEmployeeTerritoriesToExcel(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/employeeterritories/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/employeeterritories/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        public async Task ExportEmployeeTerritoriesToCSV(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/employeeterritories/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/employeeterritories/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        partial void OnEmployeeTerritoriesRead(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory> items);

        public async Task<IQueryable<Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory>> GetEmployeeTerritories(Query query = null)
        {
            var items = Context.EmployeeTerritories.AsQueryable();

            items = items.Include(i => i.Employee);
            items = items.Include(i => i.Territory);

            if (query != null)
            {
                if (!string.IsNullOrEmpty(query.Expand))
                {
                    var propertiesToExpand = query.Expand.Split(',');
                    foreach(var p in propertiesToExpand)
                    {
                        items = items.Include(p.Trim());
                    }
                }

                ApplyQuery(ref items, query);
            }

            OnEmployeeTerritoriesRead(ref items);

            return await Task.FromResult(items);
        }

        partial void OnEmployeeTerritoryGet(Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory item);
        partial void OnGetEmployeeTerritoryByEmployeeIdAndTerritoryId(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory> items);


        public async Task<Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory> GetEmployeeTerritoryByEmployeeIdAndTerritoryId(int employeeid, string territoryid)
        {
            var items = Context.EmployeeTerritories
                              .AsNoTracking()
                              .Where(i => i.EmployeeID == employeeid && i.TerritoryID == territoryid);

            items = items.Include(i => i.Employee);
            items = items.Include(i => i.Territory);
 
            OnGetEmployeeTerritoryByEmployeeIdAndTerritoryId(ref items);

            var itemToReturn = items.FirstOrDefault();

            OnEmployeeTerritoryGet(itemToReturn);

            return await Task.FromResult(itemToReturn);
        }

        partial void OnEmployeeTerritoryCreated(Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory item);
        partial void OnAfterEmployeeTerritoryCreated(Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory> CreateEmployeeTerritory(Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory employeeterritory)
        {
            OnEmployeeTerritoryCreated(employeeterritory);

            var existingItem = Context.EmployeeTerritories
                              .Where(i => i.EmployeeID == employeeterritory.EmployeeID && i.TerritoryID == employeeterritory.TerritoryID)
                              .FirstOrDefault();

            if (existingItem != null)
            {
               throw new Exception("Item already available");
            }            

            try
            {
                Context.EmployeeTerritories.Add(employeeterritory);
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(employeeterritory).State = EntityState.Detached;
                throw;
            }

            OnAfterEmployeeTerritoryCreated(employeeterritory);

            return employeeterritory;
        }

        public async Task<Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory> CancelEmployeeTerritoryChanges(Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory item)
        {
            var entityToCancel = Context.Entry(item);
            if (entityToCancel.State == EntityState.Modified)
            {
              entityToCancel.CurrentValues.SetValues(entityToCancel.OriginalValues);
              entityToCancel.State = EntityState.Unchanged;
            }

            return item;
        }

        partial void OnEmployeeTerritoryUpdated(Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory item);
        partial void OnAfterEmployeeTerritoryUpdated(Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory> UpdateEmployeeTerritory(int employeeid, string territoryid, Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory employeeterritory)
        {
            OnEmployeeTerritoryUpdated(employeeterritory);

            var itemToUpdate = Context.EmployeeTerritories
                              .Where(i => i.EmployeeID == employeeterritory.EmployeeID && i.TerritoryID == employeeterritory.TerritoryID)
                              .FirstOrDefault();

            if (itemToUpdate == null)
            {
               throw new Exception("Item no longer available");
            }
                
            var entryToUpdate = Context.Entry(itemToUpdate);
            entryToUpdate.CurrentValues.SetValues(employeeterritory);
            entryToUpdate.State = EntityState.Modified;

            Context.SaveChanges();

            OnAfterEmployeeTerritoryUpdated(employeeterritory);

            return employeeterritory;
        }

        partial void OnEmployeeTerritoryDeleted(Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory item);
        partial void OnAfterEmployeeTerritoryDeleted(Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.EmployeeTerritory> DeleteEmployeeTerritory(int employeeid, string territoryid)
        {
            var itemToDelete = Context.EmployeeTerritories
                              .Where(i => i.EmployeeID == employeeid && i.TerritoryID == territoryid)
                              .FirstOrDefault();

            if (itemToDelete == null)
            {
               throw new Exception("Item no longer available");
            }

            OnEmployeeTerritoryDeleted(itemToDelete);


            Context.EmployeeTerritories.Remove(itemToDelete);

            try
            {
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(itemToDelete).State = EntityState.Unchanged;
                throw;
            }

            OnAfterEmployeeTerritoryDeleted(itemToDelete);

            return itemToDelete;
        }
    
        public async Task ExportOrderDetailsToExcel(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/orderdetails/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/orderdetails/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        public async Task ExportOrderDetailsToCSV(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/orderdetails/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/orderdetails/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        partial void OnOrderDetailsRead(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.OrderDetail> items);

        public async Task<IQueryable<Net8AutoSecurityExport.Models.Northwind.OrderDetail>> GetOrderDetails(Query query = null)
        {
            var items = Context.OrderDetails.AsQueryable();

            items = items.Include(i => i.Order);
            items = items.Include(i => i.Product);

            if (query != null)
            {
                if (!string.IsNullOrEmpty(query.Expand))
                {
                    var propertiesToExpand = query.Expand.Split(',');
                    foreach(var p in propertiesToExpand)
                    {
                        items = items.Include(p.Trim());
                    }
                }

                ApplyQuery(ref items, query);
            }

            OnOrderDetailsRead(ref items);

            return await Task.FromResult(items);
        }

        partial void OnOrderDetailGet(Net8AutoSecurityExport.Models.Northwind.OrderDetail item);
        partial void OnGetOrderDetailByOrderIdAndProductId(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.OrderDetail> items);


        public async Task<Net8AutoSecurityExport.Models.Northwind.OrderDetail> GetOrderDetailByOrderIdAndProductId(int orderid, int productid)
        {
            var items = Context.OrderDetails
                              .AsNoTracking()
                              .Where(i => i.OrderID == orderid && i.ProductID == productid);

            items = items.Include(i => i.Order);
            items = items.Include(i => i.Product);
 
            OnGetOrderDetailByOrderIdAndProductId(ref items);

            var itemToReturn = items.FirstOrDefault();

            OnOrderDetailGet(itemToReturn);

            return await Task.FromResult(itemToReturn);
        }

        partial void OnOrderDetailCreated(Net8AutoSecurityExport.Models.Northwind.OrderDetail item);
        partial void OnAfterOrderDetailCreated(Net8AutoSecurityExport.Models.Northwind.OrderDetail item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.OrderDetail> CreateOrderDetail(Net8AutoSecurityExport.Models.Northwind.OrderDetail orderdetail)
        {
            OnOrderDetailCreated(orderdetail);

            var existingItem = Context.OrderDetails
                              .Where(i => i.OrderID == orderdetail.OrderID && i.ProductID == orderdetail.ProductID)
                              .FirstOrDefault();

            if (existingItem != null)
            {
               throw new Exception("Item already available");
            }            

            try
            {
                Context.OrderDetails.Add(orderdetail);
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(orderdetail).State = EntityState.Detached;
                throw;
            }

            OnAfterOrderDetailCreated(orderdetail);

            return orderdetail;
        }

        public async Task<Net8AutoSecurityExport.Models.Northwind.OrderDetail> CancelOrderDetailChanges(Net8AutoSecurityExport.Models.Northwind.OrderDetail item)
        {
            var entityToCancel = Context.Entry(item);
            if (entityToCancel.State == EntityState.Modified)
            {
              entityToCancel.CurrentValues.SetValues(entityToCancel.OriginalValues);
              entityToCancel.State = EntityState.Unchanged;
            }

            return item;
        }

        partial void OnOrderDetailUpdated(Net8AutoSecurityExport.Models.Northwind.OrderDetail item);
        partial void OnAfterOrderDetailUpdated(Net8AutoSecurityExport.Models.Northwind.OrderDetail item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.OrderDetail> UpdateOrderDetail(int orderid, int productid, Net8AutoSecurityExport.Models.Northwind.OrderDetail orderdetail)
        {
            OnOrderDetailUpdated(orderdetail);

            var itemToUpdate = Context.OrderDetails
                              .Where(i => i.OrderID == orderdetail.OrderID && i.ProductID == orderdetail.ProductID)
                              .FirstOrDefault();

            if (itemToUpdate == null)
            {
               throw new Exception("Item no longer available");
            }
                
            var entryToUpdate = Context.Entry(itemToUpdate);
            entryToUpdate.CurrentValues.SetValues(orderdetail);
            entryToUpdate.State = EntityState.Modified;

            Context.SaveChanges();

            OnAfterOrderDetailUpdated(orderdetail);

            return orderdetail;
        }

        partial void OnOrderDetailDeleted(Net8AutoSecurityExport.Models.Northwind.OrderDetail item);
        partial void OnAfterOrderDetailDeleted(Net8AutoSecurityExport.Models.Northwind.OrderDetail item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.OrderDetail> DeleteOrderDetail(int orderid, int productid)
        {
            var itemToDelete = Context.OrderDetails
                              .Where(i => i.OrderID == orderid && i.ProductID == productid)
                              .FirstOrDefault();

            if (itemToDelete == null)
            {
               throw new Exception("Item no longer available");
            }

            OnOrderDetailDeleted(itemToDelete);


            Context.OrderDetails.Remove(itemToDelete);

            try
            {
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(itemToDelete).State = EntityState.Unchanged;
                throw;
            }

            OnAfterOrderDetailDeleted(itemToDelete);

            return itemToDelete;
        }
    
        public async Task ExportOrdersToExcel(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/orders/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/orders/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        public async Task ExportOrdersToCSV(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/orders/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/orders/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        partial void OnOrdersRead(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Order> items);

        public async Task<IQueryable<Net8AutoSecurityExport.Models.Northwind.Order>> GetOrders(Query query = null)
        {
            var items = Context.Orders.AsQueryable();

            items = items.Include(i => i.Customer);
            items = items.Include(i => i.Employee);
            items = items.Include(i => i.Shipper);

            if (query != null)
            {
                if (!string.IsNullOrEmpty(query.Expand))
                {
                    var propertiesToExpand = query.Expand.Split(',');
                    foreach(var p in propertiesToExpand)
                    {
                        items = items.Include(p.Trim());
                    }
                }

                ApplyQuery(ref items, query);
            }

            OnOrdersRead(ref items);

            return await Task.FromResult(items);
        }

        partial void OnOrderGet(Net8AutoSecurityExport.Models.Northwind.Order item);
        partial void OnGetOrderByOrderId(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Order> items);


        public async Task<Net8AutoSecurityExport.Models.Northwind.Order> GetOrderByOrderId(int orderid)
        {
            var items = Context.Orders
                              .AsNoTracking()
                              .Where(i => i.OrderID == orderid);

            items = items.Include(i => i.Customer);
            items = items.Include(i => i.Employee);
            items = items.Include(i => i.Shipper);
 
            OnGetOrderByOrderId(ref items);

            var itemToReturn = items.FirstOrDefault();

            OnOrderGet(itemToReturn);

            return await Task.FromResult(itemToReturn);
        }

        partial void OnOrderCreated(Net8AutoSecurityExport.Models.Northwind.Order item);
        partial void OnAfterOrderCreated(Net8AutoSecurityExport.Models.Northwind.Order item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Order> CreateOrder(Net8AutoSecurityExport.Models.Northwind.Order order)
        {
            OnOrderCreated(order);

            var existingItem = Context.Orders
                              .Where(i => i.OrderID == order.OrderID)
                              .FirstOrDefault();

            if (existingItem != null)
            {
               throw new Exception("Item already available");
            }            

            try
            {
                Context.Orders.Add(order);
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(order).State = EntityState.Detached;
                throw;
            }

            OnAfterOrderCreated(order);

            return order;
        }

        public async Task<Net8AutoSecurityExport.Models.Northwind.Order> CancelOrderChanges(Net8AutoSecurityExport.Models.Northwind.Order item)
        {
            var entityToCancel = Context.Entry(item);
            if (entityToCancel.State == EntityState.Modified)
            {
              entityToCancel.CurrentValues.SetValues(entityToCancel.OriginalValues);
              entityToCancel.State = EntityState.Unchanged;
            }

            return item;
        }

        partial void OnOrderUpdated(Net8AutoSecurityExport.Models.Northwind.Order item);
        partial void OnAfterOrderUpdated(Net8AutoSecurityExport.Models.Northwind.Order item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Order> UpdateOrder(int orderid, Net8AutoSecurityExport.Models.Northwind.Order order)
        {
            OnOrderUpdated(order);

            var itemToUpdate = Context.Orders
                              .Where(i => i.OrderID == order.OrderID)
                              .FirstOrDefault();

            if (itemToUpdate == null)
            {
               throw new Exception("Item no longer available");
            }
                
            var entryToUpdate = Context.Entry(itemToUpdate);
            entryToUpdate.CurrentValues.SetValues(order);
            entryToUpdate.State = EntityState.Modified;

            Context.SaveChanges();

            OnAfterOrderUpdated(order);

            return order;
        }

        partial void OnOrderDeleted(Net8AutoSecurityExport.Models.Northwind.Order item);
        partial void OnAfterOrderDeleted(Net8AutoSecurityExport.Models.Northwind.Order item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Order> DeleteOrder(int orderid)
        {
            var itemToDelete = Context.Orders
                              .Where(i => i.OrderID == orderid)
                              .Include(i => i.OrderDetails)
                              .FirstOrDefault();

            if (itemToDelete == null)
            {
               throw new Exception("Item no longer available");
            }

            OnOrderDeleted(itemToDelete);


            Context.Orders.Remove(itemToDelete);

            try
            {
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(itemToDelete).State = EntityState.Unchanged;
                throw;
            }

            OnAfterOrderDeleted(itemToDelete);

            return itemToDelete;
        }
    
        public async Task ExportProductsToExcel(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/products/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/products/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        public async Task ExportProductsToCSV(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/products/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/products/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        partial void OnProductsRead(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Product> items);

        public async Task<IQueryable<Net8AutoSecurityExport.Models.Northwind.Product>> GetProducts(Query query = null)
        {
            var items = Context.Products.AsQueryable();

            items = items.Include(i => i.Category);
            items = items.Include(i => i.Supplier);

            if (query != null)
            {
                if (!string.IsNullOrEmpty(query.Expand))
                {
                    var propertiesToExpand = query.Expand.Split(',');
                    foreach(var p in propertiesToExpand)
                    {
                        items = items.Include(p.Trim());
                    }
                }

                ApplyQuery(ref items, query);
            }

            OnProductsRead(ref items);

            return await Task.FromResult(items);
        }

        partial void OnProductGet(Net8AutoSecurityExport.Models.Northwind.Product item);
        partial void OnGetProductByProductId(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Product> items);


        public async Task<Net8AutoSecurityExport.Models.Northwind.Product> GetProductByProductId(int productid)
        {
            var items = Context.Products
                              .AsNoTracking()
                              .Where(i => i.ProductID == productid);

            items = items.Include(i => i.Category);
            items = items.Include(i => i.Supplier);
 
            OnGetProductByProductId(ref items);

            var itemToReturn = items.FirstOrDefault();

            OnProductGet(itemToReturn);

            return await Task.FromResult(itemToReturn);
        }

        partial void OnProductCreated(Net8AutoSecurityExport.Models.Northwind.Product item);
        partial void OnAfterProductCreated(Net8AutoSecurityExport.Models.Northwind.Product item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Product> CreateProduct(Net8AutoSecurityExport.Models.Northwind.Product product)
        {
            OnProductCreated(product);

            var existingItem = Context.Products
                              .Where(i => i.ProductID == product.ProductID)
                              .FirstOrDefault();

            if (existingItem != null)
            {
               throw new Exception("Item already available");
            }            

            try
            {
                Context.Products.Add(product);
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(product).State = EntityState.Detached;
                throw;
            }

            OnAfterProductCreated(product);

            return product;
        }

        public async Task<Net8AutoSecurityExport.Models.Northwind.Product> CancelProductChanges(Net8AutoSecurityExport.Models.Northwind.Product item)
        {
            var entityToCancel = Context.Entry(item);
            if (entityToCancel.State == EntityState.Modified)
            {
              entityToCancel.CurrentValues.SetValues(entityToCancel.OriginalValues);
              entityToCancel.State = EntityState.Unchanged;
            }

            return item;
        }

        partial void OnProductUpdated(Net8AutoSecurityExport.Models.Northwind.Product item);
        partial void OnAfterProductUpdated(Net8AutoSecurityExport.Models.Northwind.Product item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Product> UpdateProduct(int productid, Net8AutoSecurityExport.Models.Northwind.Product product)
        {
            OnProductUpdated(product);

            var itemToUpdate = Context.Products
                              .Where(i => i.ProductID == product.ProductID)
                              .FirstOrDefault();

            if (itemToUpdate == null)
            {
               throw new Exception("Item no longer available");
            }
                
            var entryToUpdate = Context.Entry(itemToUpdate);
            entryToUpdate.CurrentValues.SetValues(product);
            entryToUpdate.State = EntityState.Modified;

            Context.SaveChanges();

            OnAfterProductUpdated(product);

            return product;
        }

        partial void OnProductDeleted(Net8AutoSecurityExport.Models.Northwind.Product item);
        partial void OnAfterProductDeleted(Net8AutoSecurityExport.Models.Northwind.Product item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Product> DeleteProduct(int productid)
        {
            var itemToDelete = Context.Products
                              .Where(i => i.ProductID == productid)
                              .Include(i => i.OrderDetails)
                              .FirstOrDefault();

            if (itemToDelete == null)
            {
               throw new Exception("Item no longer available");
            }

            OnProductDeleted(itemToDelete);


            Context.Products.Remove(itemToDelete);

            try
            {
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(itemToDelete).State = EntityState.Unchanged;
                throw;
            }

            OnAfterProductDeleted(itemToDelete);

            return itemToDelete;
        }
    
        public async Task ExportRegionsToExcel(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/regions/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/regions/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        public async Task ExportRegionsToCSV(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/regions/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/regions/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        partial void OnRegionsRead(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Region> items);

        public async Task<IQueryable<Net8AutoSecurityExport.Models.Northwind.Region>> GetRegions(Query query = null)
        {
            var items = Context.Regions.AsQueryable();


            if (query != null)
            {
                if (!string.IsNullOrEmpty(query.Expand))
                {
                    var propertiesToExpand = query.Expand.Split(',');
                    foreach(var p in propertiesToExpand)
                    {
                        items = items.Include(p.Trim());
                    }
                }

                ApplyQuery(ref items, query);
            }

            OnRegionsRead(ref items);

            return await Task.FromResult(items);
        }

        partial void OnRegionGet(Net8AutoSecurityExport.Models.Northwind.Region item);
        partial void OnGetRegionByRegionId(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Region> items);


        public async Task<Net8AutoSecurityExport.Models.Northwind.Region> GetRegionByRegionId(int regionid)
        {
            var items = Context.Regions
                              .AsNoTracking()
                              .Where(i => i.RegionID == regionid);

 
            OnGetRegionByRegionId(ref items);

            var itemToReturn = items.FirstOrDefault();

            OnRegionGet(itemToReturn);

            return await Task.FromResult(itemToReturn);
        }

        partial void OnRegionCreated(Net8AutoSecurityExport.Models.Northwind.Region item);
        partial void OnAfterRegionCreated(Net8AutoSecurityExport.Models.Northwind.Region item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Region> CreateRegion(Net8AutoSecurityExport.Models.Northwind.Region region)
        {
            OnRegionCreated(region);

            var existingItem = Context.Regions
                              .Where(i => i.RegionID == region.RegionID)
                              .FirstOrDefault();

            if (existingItem != null)
            {
               throw new Exception("Item already available");
            }            

            try
            {
                Context.Regions.Add(region);
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(region).State = EntityState.Detached;
                throw;
            }

            OnAfterRegionCreated(region);

            return region;
        }

        public async Task<Net8AutoSecurityExport.Models.Northwind.Region> CancelRegionChanges(Net8AutoSecurityExport.Models.Northwind.Region item)
        {
            var entityToCancel = Context.Entry(item);
            if (entityToCancel.State == EntityState.Modified)
            {
              entityToCancel.CurrentValues.SetValues(entityToCancel.OriginalValues);
              entityToCancel.State = EntityState.Unchanged;
            }

            return item;
        }

        partial void OnRegionUpdated(Net8AutoSecurityExport.Models.Northwind.Region item);
        partial void OnAfterRegionUpdated(Net8AutoSecurityExport.Models.Northwind.Region item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Region> UpdateRegion(int regionid, Net8AutoSecurityExport.Models.Northwind.Region region)
        {
            OnRegionUpdated(region);

            var itemToUpdate = Context.Regions
                              .Where(i => i.RegionID == region.RegionID)
                              .FirstOrDefault();

            if (itemToUpdate == null)
            {
               throw new Exception("Item no longer available");
            }
                
            var entryToUpdate = Context.Entry(itemToUpdate);
            entryToUpdate.CurrentValues.SetValues(region);
            entryToUpdate.State = EntityState.Modified;

            Context.SaveChanges();

            OnAfterRegionUpdated(region);

            return region;
        }

        partial void OnRegionDeleted(Net8AutoSecurityExport.Models.Northwind.Region item);
        partial void OnAfterRegionDeleted(Net8AutoSecurityExport.Models.Northwind.Region item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Region> DeleteRegion(int regionid)
        {
            var itemToDelete = Context.Regions
                              .Where(i => i.RegionID == regionid)
                              .Include(i => i.Territories)
                              .FirstOrDefault();

            if (itemToDelete == null)
            {
               throw new Exception("Item no longer available");
            }

            OnRegionDeleted(itemToDelete);


            Context.Regions.Remove(itemToDelete);

            try
            {
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(itemToDelete).State = EntityState.Unchanged;
                throw;
            }

            OnAfterRegionDeleted(itemToDelete);

            return itemToDelete;
        }
    
        public async Task ExportShippersToExcel(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/shippers/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/shippers/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        public async Task ExportShippersToCSV(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/shippers/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/shippers/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        partial void OnShippersRead(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Shipper> items);

        public async Task<IQueryable<Net8AutoSecurityExport.Models.Northwind.Shipper>> GetShippers(Query query = null)
        {
            var items = Context.Shippers.AsQueryable();


            if (query != null)
            {
                if (!string.IsNullOrEmpty(query.Expand))
                {
                    var propertiesToExpand = query.Expand.Split(',');
                    foreach(var p in propertiesToExpand)
                    {
                        items = items.Include(p.Trim());
                    }
                }

                ApplyQuery(ref items, query);
            }

            OnShippersRead(ref items);

            return await Task.FromResult(items);
        }

        partial void OnShipperGet(Net8AutoSecurityExport.Models.Northwind.Shipper item);
        partial void OnGetShipperByShipperId(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Shipper> items);


        public async Task<Net8AutoSecurityExport.Models.Northwind.Shipper> GetShipperByShipperId(int shipperid)
        {
            var items = Context.Shippers
                              .AsNoTracking()
                              .Where(i => i.ShipperID == shipperid);

 
            OnGetShipperByShipperId(ref items);

            var itemToReturn = items.FirstOrDefault();

            OnShipperGet(itemToReturn);

            return await Task.FromResult(itemToReturn);
        }

        partial void OnShipperCreated(Net8AutoSecurityExport.Models.Northwind.Shipper item);
        partial void OnAfterShipperCreated(Net8AutoSecurityExport.Models.Northwind.Shipper item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Shipper> CreateShipper(Net8AutoSecurityExport.Models.Northwind.Shipper shipper)
        {
            OnShipperCreated(shipper);

            var existingItem = Context.Shippers
                              .Where(i => i.ShipperID == shipper.ShipperID)
                              .FirstOrDefault();

            if (existingItem != null)
            {
               throw new Exception("Item already available");
            }            

            try
            {
                Context.Shippers.Add(shipper);
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(shipper).State = EntityState.Detached;
                throw;
            }

            OnAfterShipperCreated(shipper);

            return shipper;
        }

        public async Task<Net8AutoSecurityExport.Models.Northwind.Shipper> CancelShipperChanges(Net8AutoSecurityExport.Models.Northwind.Shipper item)
        {
            var entityToCancel = Context.Entry(item);
            if (entityToCancel.State == EntityState.Modified)
            {
              entityToCancel.CurrentValues.SetValues(entityToCancel.OriginalValues);
              entityToCancel.State = EntityState.Unchanged;
            }

            return item;
        }

        partial void OnShipperUpdated(Net8AutoSecurityExport.Models.Northwind.Shipper item);
        partial void OnAfterShipperUpdated(Net8AutoSecurityExport.Models.Northwind.Shipper item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Shipper> UpdateShipper(int shipperid, Net8AutoSecurityExport.Models.Northwind.Shipper shipper)
        {
            OnShipperUpdated(shipper);

            var itemToUpdate = Context.Shippers
                              .Where(i => i.ShipperID == shipper.ShipperID)
                              .FirstOrDefault();

            if (itemToUpdate == null)
            {
               throw new Exception("Item no longer available");
            }
                
            var entryToUpdate = Context.Entry(itemToUpdate);
            entryToUpdate.CurrentValues.SetValues(shipper);
            entryToUpdate.State = EntityState.Modified;

            Context.SaveChanges();

            OnAfterShipperUpdated(shipper);

            return shipper;
        }

        partial void OnShipperDeleted(Net8AutoSecurityExport.Models.Northwind.Shipper item);
        partial void OnAfterShipperDeleted(Net8AutoSecurityExport.Models.Northwind.Shipper item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Shipper> DeleteShipper(int shipperid)
        {
            var itemToDelete = Context.Shippers
                              .Where(i => i.ShipperID == shipperid)
                              .Include(i => i.Orders)
                              .FirstOrDefault();

            if (itemToDelete == null)
            {
               throw new Exception("Item no longer available");
            }

            OnShipperDeleted(itemToDelete);


            Context.Shippers.Remove(itemToDelete);

            try
            {
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(itemToDelete).State = EntityState.Unchanged;
                throw;
            }

            OnAfterShipperDeleted(itemToDelete);

            return itemToDelete;
        }
    
        public async Task ExportSuppliersToExcel(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/suppliers/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/suppliers/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        public async Task ExportSuppliersToCSV(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/suppliers/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/suppliers/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        partial void OnSuppliersRead(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Supplier> items);

        public async Task<IQueryable<Net8AutoSecurityExport.Models.Northwind.Supplier>> GetSuppliers(Query query = null)
        {
            var items = Context.Suppliers.AsQueryable();


            if (query != null)
            {
                if (!string.IsNullOrEmpty(query.Expand))
                {
                    var propertiesToExpand = query.Expand.Split(',');
                    foreach(var p in propertiesToExpand)
                    {
                        items = items.Include(p.Trim());
                    }
                }

                ApplyQuery(ref items, query);
            }

            OnSuppliersRead(ref items);

            return await Task.FromResult(items);
        }

        partial void OnSupplierGet(Net8AutoSecurityExport.Models.Northwind.Supplier item);
        partial void OnGetSupplierBySupplierId(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Supplier> items);


        public async Task<Net8AutoSecurityExport.Models.Northwind.Supplier> GetSupplierBySupplierId(int supplierid)
        {
            var items = Context.Suppliers
                              .AsNoTracking()
                              .Where(i => i.SupplierID == supplierid);

 
            OnGetSupplierBySupplierId(ref items);

            var itemToReturn = items.FirstOrDefault();

            OnSupplierGet(itemToReturn);

            return await Task.FromResult(itemToReturn);
        }

        partial void OnSupplierCreated(Net8AutoSecurityExport.Models.Northwind.Supplier item);
        partial void OnAfterSupplierCreated(Net8AutoSecurityExport.Models.Northwind.Supplier item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Supplier> CreateSupplier(Net8AutoSecurityExport.Models.Northwind.Supplier supplier)
        {
            OnSupplierCreated(supplier);

            var existingItem = Context.Suppliers
                              .Where(i => i.SupplierID == supplier.SupplierID)
                              .FirstOrDefault();

            if (existingItem != null)
            {
               throw new Exception("Item already available");
            }            

            try
            {
                Context.Suppliers.Add(supplier);
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(supplier).State = EntityState.Detached;
                throw;
            }

            OnAfterSupplierCreated(supplier);

            return supplier;
        }

        public async Task<Net8AutoSecurityExport.Models.Northwind.Supplier> CancelSupplierChanges(Net8AutoSecurityExport.Models.Northwind.Supplier item)
        {
            var entityToCancel = Context.Entry(item);
            if (entityToCancel.State == EntityState.Modified)
            {
              entityToCancel.CurrentValues.SetValues(entityToCancel.OriginalValues);
              entityToCancel.State = EntityState.Unchanged;
            }

            return item;
        }

        partial void OnSupplierUpdated(Net8AutoSecurityExport.Models.Northwind.Supplier item);
        partial void OnAfterSupplierUpdated(Net8AutoSecurityExport.Models.Northwind.Supplier item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Supplier> UpdateSupplier(int supplierid, Net8AutoSecurityExport.Models.Northwind.Supplier supplier)
        {
            OnSupplierUpdated(supplier);

            var itemToUpdate = Context.Suppliers
                              .Where(i => i.SupplierID == supplier.SupplierID)
                              .FirstOrDefault();

            if (itemToUpdate == null)
            {
               throw new Exception("Item no longer available");
            }
                
            var entryToUpdate = Context.Entry(itemToUpdate);
            entryToUpdate.CurrentValues.SetValues(supplier);
            entryToUpdate.State = EntityState.Modified;

            Context.SaveChanges();

            OnAfterSupplierUpdated(supplier);

            return supplier;
        }

        partial void OnSupplierDeleted(Net8AutoSecurityExport.Models.Northwind.Supplier item);
        partial void OnAfterSupplierDeleted(Net8AutoSecurityExport.Models.Northwind.Supplier item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Supplier> DeleteSupplier(int supplierid)
        {
            var itemToDelete = Context.Suppliers
                              .Where(i => i.SupplierID == supplierid)
                              .Include(i => i.Products)
                              .FirstOrDefault();

            if (itemToDelete == null)
            {
               throw new Exception("Item no longer available");
            }

            OnSupplierDeleted(itemToDelete);


            Context.Suppliers.Remove(itemToDelete);

            try
            {
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(itemToDelete).State = EntityState.Unchanged;
                throw;
            }

            OnAfterSupplierDeleted(itemToDelete);

            return itemToDelete;
        }
    
        public async Task ExportTerritoriesToExcel(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/territories/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/territories/excel(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        public async Task ExportTerritoriesToCSV(Query query = null, string fileName = null)
        {
            navigationManager.NavigateTo(query != null ? query.ToUrl($"export/northwind/territories/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')") : $"export/northwind/territories/csv(fileName='{(!string.IsNullOrEmpty(fileName) ? UrlEncoder.Default.Encode(fileName) : "Export")}')", true);
        }

        partial void OnTerritoriesRead(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Territory> items);

        public async Task<IQueryable<Net8AutoSecurityExport.Models.Northwind.Territory>> GetTerritories(Query query = null)
        {
            var items = Context.Territories.AsQueryable();

            items = items.Include(i => i.Region);

            if (query != null)
            {
                if (!string.IsNullOrEmpty(query.Expand))
                {
                    var propertiesToExpand = query.Expand.Split(',');
                    foreach(var p in propertiesToExpand)
                    {
                        items = items.Include(p.Trim());
                    }
                }

                ApplyQuery(ref items, query);
            }

            OnTerritoriesRead(ref items);

            return await Task.FromResult(items);
        }

        partial void OnTerritoryGet(Net8AutoSecurityExport.Models.Northwind.Territory item);
        partial void OnGetTerritoryByTerritoryId(ref IQueryable<Net8AutoSecurityExport.Models.Northwind.Territory> items);


        public async Task<Net8AutoSecurityExport.Models.Northwind.Territory> GetTerritoryByTerritoryId(string territoryid)
        {
            var items = Context.Territories
                              .AsNoTracking()
                              .Where(i => i.TerritoryID == territoryid);

            items = items.Include(i => i.Region);
 
            OnGetTerritoryByTerritoryId(ref items);

            var itemToReturn = items.FirstOrDefault();

            OnTerritoryGet(itemToReturn);

            return await Task.FromResult(itemToReturn);
        }

        partial void OnTerritoryCreated(Net8AutoSecurityExport.Models.Northwind.Territory item);
        partial void OnAfterTerritoryCreated(Net8AutoSecurityExport.Models.Northwind.Territory item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Territory> CreateTerritory(Net8AutoSecurityExport.Models.Northwind.Territory territory)
        {
            OnTerritoryCreated(territory);

            var existingItem = Context.Territories
                              .Where(i => i.TerritoryID == territory.TerritoryID)
                              .FirstOrDefault();

            if (existingItem != null)
            {
               throw new Exception("Item already available");
            }            

            try
            {
                Context.Territories.Add(territory);
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(territory).State = EntityState.Detached;
                throw;
            }

            OnAfterTerritoryCreated(territory);

            return territory;
        }

        public async Task<Net8AutoSecurityExport.Models.Northwind.Territory> CancelTerritoryChanges(Net8AutoSecurityExport.Models.Northwind.Territory item)
        {
            var entityToCancel = Context.Entry(item);
            if (entityToCancel.State == EntityState.Modified)
            {
              entityToCancel.CurrentValues.SetValues(entityToCancel.OriginalValues);
              entityToCancel.State = EntityState.Unchanged;
            }

            return item;
        }

        partial void OnTerritoryUpdated(Net8AutoSecurityExport.Models.Northwind.Territory item);
        partial void OnAfterTerritoryUpdated(Net8AutoSecurityExport.Models.Northwind.Territory item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Territory> UpdateTerritory(string territoryid, Net8AutoSecurityExport.Models.Northwind.Territory territory)
        {
            OnTerritoryUpdated(territory);

            var itemToUpdate = Context.Territories
                              .Where(i => i.TerritoryID == territory.TerritoryID)
                              .FirstOrDefault();

            if (itemToUpdate == null)
            {
               throw new Exception("Item no longer available");
            }
                
            var entryToUpdate = Context.Entry(itemToUpdate);
            entryToUpdate.CurrentValues.SetValues(territory);
            entryToUpdate.State = EntityState.Modified;

            Context.SaveChanges();

            OnAfterTerritoryUpdated(territory);

            return territory;
        }

        partial void OnTerritoryDeleted(Net8AutoSecurityExport.Models.Northwind.Territory item);
        partial void OnAfterTerritoryDeleted(Net8AutoSecurityExport.Models.Northwind.Territory item);

        public async Task<Net8AutoSecurityExport.Models.Northwind.Territory> DeleteTerritory(string territoryid)
        {
            var itemToDelete = Context.Territories
                              .Where(i => i.TerritoryID == territoryid)
                              .Include(i => i.EmployeeTerritories)
                              .FirstOrDefault();

            if (itemToDelete == null)
            {
               throw new Exception("Item no longer available");
            }

            OnTerritoryDeleted(itemToDelete);


            Context.Territories.Remove(itemToDelete);

            try
            {
                Context.SaveChanges();
            }
            catch
            {
                Context.Entry(itemToDelete).State = EntityState.Unchanged;
                throw;
            }

            OnAfterTerritoryDeleted(itemToDelete);

            return itemToDelete;
        }
        }
}